@@ -3,12 +3,13 @@ __author__ = "Thomas Baldauf"
 __email__ = "thomas.baldauf@dlr.de"
 __license__ = "MIT"
 __birthdate__ = '15.11.2021'
-__status__ = 'prod' # options are: dev, test, prod
+__status__ = 'prod'  # options are: dev, test, prod
 
 from .singleton import Singleton
 from datetime import datetime, timedelta
 from dateutil.relativedelta import relativedelta
 import warnings
+import time
 
 
 class Clock(Singleton):
@@ -16,7 +17,7 @@ class Clock(Singleton):
     The clock of the simulation.
     """
 
-    def __init__(self, t0=datetime.min, dt=relativedelta(months=1)):
+    def __init__(self, t0=datetime.min, dt=relativedelta(years=1)):
         """
         Init a world clock.
 
@@ -37,19 +38,33 @@ class Clock(Singleton):
 
         """
 
-        if self.do_init: # only init the clock at the first constructor call! See Singleton super class implementation
+        if not self.is_initialized:
+            super().__init__()
 
-            self._t0 = t0
+            self._t0 = t0  # datetime.min
             self._t = 0  # the simulation tme
             self._real_time = t0  # the real time as datetime object starting at big bang
-            self._dt = dt
-            self.set_initialized()
+            self._dt = dt  # relativedelta(years=1)
+            self._delay = 0  # artificial delay in the iterations
+
+    def set_delay(self, delay=0.001):
+        """
+        Insert an artificial delay when doing Clock().tick() 
+
+        Args:
+            delay (float): seconds delay
+        """
+        self._delay = delay
+
+    @property
+    def dt(self):
+        return self._dt
 
     @property
     def t0(self):
         return self._t0
 
-    def reset(self,verbose=False):
+    def reset(self, verbose=False):
         """
         Resets the clock
 
@@ -62,6 +77,7 @@ class Clock(Singleton):
         self._t = 0  # the simulation tme
         self._real_time = self._t0  # the real time as datetime object starting at big bang
 
+
     def tick(self):
         """
         Increases the clock tick by one
@@ -69,6 +85,14 @@ class Clock(Singleton):
         self._t += 1
         self._real_time += self._dt
 
+        if self._delay > 0:
+            time.sleep(self._delay)
+
+        return self._t
+
+    def time(self):
+        return self.get_time()
+
     def get_time(self):
         """
         Gets the current simulation time as int
