@@ -11,5 +11,5 @@ def warning_on_one_line(message, category, filename, lineno, file=None, line=Non
     """
     This defines the warning format of all sfctools warnings.
     """
-    return '%s %s: %s [%s LINE %i]\n' % (":-( sfctools", category.__name__, message,os.path.basename(filename),lineno)
+    return '%s %s: %s [%s LINE %i]\n' % ("[sfctools] ", category.__name__, message,os.path.basename(filename),lineno)
 
