@@ -6,7 +6,6 @@ __birthdate__ = '15.11.2021'
 __status__ = 'dev' # options are: dev, test, prod
 
 
-
 from collections import defaultdict
 import numpy as np
 
@@ -42,6 +41,13 @@ class WorkerRegistry:
 
         self.wage_attr = wage_attr
 
+    @property 
+    def workers(self):
+        """
+        returns a list of workers
+        """
+        return self.agent_data
+
     def log_wage_bill(self,t, wb):
         """
         stores a value for wage bill wb at time t
