@@ -5,6 +5,7 @@ __license__ = "MIT"
 __birthdate__ = '15.11.2021'
 __status__ = 'dev' # options are: dev, test, prod
 
+import pandas as pd
 import numpy as np
 import warnings
 
@@ -25,13 +26,16 @@ class IndicatorReport:
 
     _instances = {}
 
-    def __init__(self,xlabel,ylabel):
+    def __init__(self,xlabel,ylabel,data=None,name=None):
         """
-
         :param xlabel: x axis label, str
         :param ylabel: y axis label, str
+        :param data: some initial data (default None). Has to have 'append' method
         """
-        self.data = []
+        if data is not None:
+            self.data = data
+        else:
+            self.data = []
         self.xlabel = xlabel
         self.ylabel = ylabel
 
@@ -39,6 +43,14 @@ class IndicatorReport:
             self.__class__._instances[self.ylabel] = self
         else:
             warnings.warn("Tried to add a report with a variable that is already registered. Skipping...")
+    
+        if name is None:
+            self.name = "Report"
+        else:
+            self.name = name 
+
+    def __str__(self):
+        return "%s (%s, %s)" % (self.name, self.xlabel, self.ylabel)
 
     @classmethod
     def getitem(cls,key):
@@ -57,7 +69,7 @@ class IndicatorReport:
         """
         self.data.append(x)
 
-    def plot_data(self,ax):
+    def plot_data(self,ax,s=1.8,color="black",scatter_color="gray"):
         """
         plots the data onto a matplotlib axis
 
@@ -71,10 +83,10 @@ class IndicatorReport:
         if len(data) > 1 and data[1] is not None:
             has_labels = True
 
-        if len(y) < 100:
-            ax.scatter(x, y, color="gray", s=4.0)  # <- manually optimized
-        else:
-            ax.plot(x, y, "-", color="black") # <- manually optimized
+        ax.plot(x, y, "-", color=color) # <- manually optimized
+
+        if len(y) < 20:
+            ax.scatter(x, y, color=scatter_color, s=s)  # <- manually optimized
 
         ax.set_xlabel(self.xlabel)
         ax.set_ylabel(self.ylabel)
@@ -87,15 +99,31 @@ class DistributionReport:
     A generic class for a logger of a distribution (i.e. )
     """
 
-    def __init__(self,title):
+    def __init__(self,xlabel,ylabel,data=None, name=None):
         """
-        :param title: title of the report
+        :param xlabel: x axis title of the report
+        :param ylabel: y axis title of the report
+        :param data: initial data (default None). Has to have 'append' method, ideally a list of tuples (data_i, tag)
         """
+        if data is not None:
+            sorted_data = list(data.copy())
+            sorted_data.sort(key = lambda x: x[0])
+            self.data = sorted_data
+        else:
+            self.data = []
 
-        self.data = []
-        self.title = title
+        self.xlabel = xlabel
+        self.ylabel = ylabel
 
-    def add_data(self, x, label=None):
+        if name is None:
+            self.name = "Report"
+        else:
+            self.name = name 
+
+    def __str__(self):
+        return "%s (%s, %s)" % (self.name, self.xlabel, self.ylabel)
+
+    def add_data(self, x, sort = True, label=None):
         """
         adds data into the data structure. (stores a sorted version of x).
 
@@ -103,45 +131,56 @@ class DistributionReport:
         :param label: some additional tag
         """
 
-        y = np.sort(x.copy())
+        if sort:
+            y = np.sort(x.copy())
+        else:
+            y = x.copy()
+
+        if label is None:
+            label = list(range(len(y)))
 
         # store distribution
-        self.data.append((y,label))
+        self.data.append((label,y))
 
-    def plot_data(self, ax,colors=None):
+    def plot_data(self, ax,color=None,s = None):
         """
         plot the data onto a matplotlib axis
 
         :param ax: a matplotlib axis
-        :param colors: list of colors to use (default None)
+        :param color: plotting color
+        :param s: scatter size
         """
 
-        colors = ["gray", "indianred", "mediumblue"]
+        if color is None:
+            col = "gray" # colors = ["gray", "indianred", "mediumblue"]
+        else:
+            col = color
 
-        has_labels = False
+        sorted_data = list(self.data.copy())
+        sorted_data.sort(key = lambda x: x[0])
 
-        for i,data in enumerate(self.data):
-            x = np.arange(len(data[0]))+1
-            y = data[0] # np.random.rand(len(x))
-            label = data[1] or ""
-            if data[1] is not None:
-                has_labels = True
-            ax.plot(x, y,color=colors[i%len(colors)], alpha=1-0.8*i/len(self.data[0]),label=label)
-            ax.scatter(x, y, color=colors[i % len(colors)], s=.5)
+        if s is None:
+            s = max(2.0, 8.0 - 4.0 * (len(sorted_data)/1000.0))
 
-        ax.set_ylabel(self.title)
-        ax.set_xlabel("Rank")
+        for i, data in enumerate(sorted_data):
+            x = data[0]
+            y = data[1] # np.random.rand(len(x))
 
-        if has_labels:
-            ax.legend(bbox_to_anchor=(1.05, 1), loc=2, borderaxespad=0.)
+            # #ax.plot(x, y,color=colors[i%len(colors)], alpha=1-0.8*i/len(self.data[0]),label=label)
+            ax.scatter(x, y, color=col, s= s)
+
+        ax.set_ylabel(self.ylabel)
+        ax.set_xlabel(self.xlabel)
+        # ax.legend(bbox_to_anchor=(1.05, 1), loc=2, borderaxespad=0.)
 
 
 class ReportingSheet:
     """
     Reporting sheet is an overview sheet for reporting model results in form of a grid plot.
     """
+    instances = []
 
-    def __init__(self,instances=None):
+    def __init__(self,instances=None,name=None):
         """
         :param instances: default None, if not None: list of Report instances can be passed here
         """
@@ -150,29 +189,56 @@ class ReportingSheet:
         else:
             self.items = instances
 
+        self.__class__.instances.append(self)
+
+        if name is None:
+            self.name = "Report_%03i" % len(self.__class__.instances)
+        else:
+            self.name = str(name)
+
     def add_report(self,report):
         """
         Adds a report item to this sheet.
 
         :param report: any IndicatorReport or DistributionReport
         """
-        self.items.append(report)
+        if report not in self.items:
+            self.items.append(report)
+    
+    def remove_report(self, report):
+        """
+        Removes a report item to this sheet (if exists)
 
-    def plot(self, show_plot=True):
+        :param report: any IndicatorReport or DistributionReport
+        """
+        if report in self.items:
+            self.items.remove(report)
+    
+
+    def plot(self, show_plot=True,verbose=False):
         """
         Generates a plot grid using matplotlib
 
         :param show_plot: show the figure in a window? default True. If False, figure is returned
+        :param verbose: print number of rows and columns if True (default False)
         :return fig: matplotlib figure
         """
+
         import matplotlib.pyplot as plt
         plt.rcParams.update({'font.size': 8})
 
         # layout is 2 plots per column...
-        n_cols = 2
-        n_rows = max(1,int(np.ceil(len(self.items)/n_cols)))
+        
+        if len(self.items) == 1:
+            n_cols = 1
+            n_rows = 1
+
+        else:
+            n_cols = 2
+            n_rows = max(1,int(np.ceil(len(self.items)/n_cols)))
+
+        if verbose: print("<Reporting Sheet> n_rows","n_cols",n_rows,n_cols)
 
-        print("n_rows","n_cols",n_rows,n_cols)
         cm = 1 / 2.54  # centimeters in inches
         fig = plt.figure(figsize=(0.5*3.1415*10.5*cm,14.8*cm)) # DIN A6
 
@@ -186,13 +252,34 @@ class ReportingSheet:
             item.plot_data(ax)
 
         plt.tight_layout()
-        fig.subplots_adjust(hspace=.67)
+        fig.subplots_adjust(hspace=.47)
 
         if show_plot:
             plt.show()
 
         return fig
 
+    def to_dataframe(self):
+        """
+        combines the data into a single pandas dataframe
+        """
+        df = pd.DataFrame()
+        for i, item in enumerate(self.items):
+            # TODO warn if something is overwritten
+
+            if isinstance(item, IndicatorReport):
+                df[item.xlabel] = list(range(len(item.data)))
+                df[item.ylabel] = item.data
+
+            elif isinstance(item, DistributionReport):
+                df[item.xlabel] = [d[0] for d in item.data]
+                df[item.ylabel] = [d[1] for d in item.data]
+
+            else:
+                warnigns.warn("Found strang item in Reporting sheet. Skipping. %s" % str(item))
+        # df.index.name = self.items[0].xlabel
+        return df
+
     def to_latex(self):
         """
         Generates latex code NOT YET IMPLEMENTED
